--
-- Diese Datei enthält Tabellen und Funktionen, die als Vorbedingung existieren
-- müssen bevor Tabellen aus dem Formed erstellt werden könn. Desweiteren
-- werden ein Teil der Berechtigungen angepasst, die beim Kopieren der
-- Tabellen aus der Templatedatenbank nicht mehr korrekt sind.
--

SET ROLE postgres;

ALTER FUNCTION waska_revoke_connect( varchar, varchar ) OWNER TO :adm_ka_owner;
GRANT EXECUTE ON FUNCTION  waska_revoke_connect( varchar, varchar ) 
    TO GROUP waska_role_adm;

REVOKE ALL ON SCHEMA public FROM PUBLIC;
GRANT USAGE ON SCHEMA public TO :cm_ka_group;
GRANT USAGE ON SCHEMA public TO :admin_ka_group;
GRANT USAGE ON SCHEMA public TO waska_role_adm;
GRANT ALL ON SCHEMA public TO :adm_ka_owner;

SET ROLE :adm_ka_owner;

BEGIN;
--
-- Init logging of DB updates.
--
CREATE TABLE db_history (
        id SERIAL PRIMARY KEY, 
        beschreibung VARCHAR(80), 
        zeitpunkt TIMESTAMP);
REVOKE ALL ON db_history FROM public;

INSERT INTO db_history (beschreibung, zeitpunkt) 
	VALUES ('mpuls WASKA DB $Revision$', current_timestamp);

-- ka_benutzergruppe_tbl
-- Tabelle fuer die Benutzergruppen. Jeder Benutzer erhaelt eine
-- Benutzergruppe in der er exklusiv Mitglied ist. 
-- Weitere Benutzergruppen können angelegt werden, die weitere Nutzer
-- enthalten können

CREATE TABLE ka_benutzergruppe_tbl
(
	id serial NOT NULL,
	name varchar(128) NOT NULL,
	CONSTRAINT ka_benutzergruppe_tbl_pkey PRIMARY KEY (id)
);

-- Benutzertabelle
-- Diese Tabelle enthält erweiterte Angaben zu den Nutzern der KA Datenbank.
-- Das Feld 'login' enthält hier den vollen Login-Namen, so wie er clusterweit
-- in der Datenbank bekannt ist (ka_<agentur_<loginname>)
-- 
CREATE TABLE ka_benutzer_tbl
(
	id serial NOT NULL,
	login varchar(64) NOT NULL,
	vorname varchar(64),
	nachname varchar(64),
	telefon varchar(32),
	raumnummer varchar(16),
	filiale varchar(64),
	aktiviert bool DEFAULT true,
	passwort_aendern bool DEFAULT true,
	last_login timestamp,
	gid       int4 REFERENCES ka_benutzergruppe_tbl(id) ON DELETE CASCADE,
	vertreter int4 REFERENCES ka_benutzergruppe_tbl(id) ON DELETE SET DEFAULT,
	CONSTRAINT ka_benutzer_tbl_pkey PRIMARY KEY (id)
); 

GRANT SELECT ON ka_benutzer_tbl TO GROUP :cm_ka_group;
GRANT SELECT ON ka_benutzer_tbl TO GROUP :admin_ka_group;
GRANT UPDATE ON ka_benutzer_tbl TO GROUP :admin_ka_group;

--
-- Benutzer-Fallakte Transitionstabelle 
-- Diese Tabelle verbindet Nutzer mit Fallakten, und legt auf diese Weise
-- fest welche Benutzer ausser dem initialen Bearbeiter oder dem KA-Admin 
-- Zugriff auf die Fallakte haben.
-- 

CREATE TABLE nm_benutzer_master_tbl (
    id serial PRIMARY KEY,
    benutzer_id int4,
    master_id int4
);

GRANT SELECT ON ka_benutzer_tbl TO GROUP :cm_ka_group;
GRANT SELECT ON ka_benutzer_tbl TO GROUP :admin_ka_group;

-- ka_nm_benutzer_gruppe_tbl
-- Tabelle zur Zuordnung zwischen Benutzern und Benutzergruppen.
CREATE TABLE nm_benutzer_gruppe_tbl
(
	id serial NOT NULL,
	gruppe_id int4 REFERENCES ka_benutzergruppe_tbl(id) ON DELETE CASCADE,
	benutzer_id int4 REFERENCES ka_benutzer_tbl(id) ON DELETE CASCADE,
	CONSTRAINT nm_benutzer_gruppe_tbl_pkey PRIMARY KEY (id)
);

--
-- getuserid()
-- holt id des current_user aus der ka_benutzer_tbl
--
CREATE OR REPLACE FUNCTION getuserid() RETURNS integer
    LANGUAGE 'plpgsql' STABLE
    AS $$
DECLARE
myid integer;
BEGIN
SELECT INTO myid id FROM ka_benutzer_tbl WHERE login = session_user;
RETURN myid;
END;
$$;

REVOKE ALL ON FUNCTION getuserid() FROM PUBLIC;
GRANT EXECUTE ON FUNCTION getuserid() TO GROUP :cm_ka_group;
GRANT EXECUTE ON FUNCTION getuserid() TO GROUP :admin_ka_group;

-- Need to be implemented.
--CREATE TABLE ka_termine_tbl;
--CREATE TABLE ka_dokumente_tbl;

-- Author: Torsten Irlaender
-- Funktionen, die für die Anonymisierung der Fallakten genutzt werden.

CREATE OR REPLACE FUNCTION shorten_value(myvalue varchar, len integer) 
RETURNS varchar 
LANGUAGE 'plpgsql' STABLE AS 
$$
BEGIN
    return substring(myvalue, 1, len);
END;
$$;

REVOKE ALL ON FUNCTION shorten_value(varchar, integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION shorten_value(varchar, integer) TO GROUP :cm_ka_group;
GRANT EXECUTE ON FUNCTION shorten_value(varchar, integer) TO GROUP :admin_ka_group;

CREATE OR REPLACE FUNCTION hash_value(myvalue varchar, myvalue2 varchar) 
RETURNS varchar 
LANGUAGE 'plpgsql' STABLE AS 
$$
BEGIN
    return md5(coalesce(myvalue, '') || coalesce(myvalue2, ''));
END;
$$;

REVOKE ALL ON FUNCTION hash_value(varchar, varchar) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION hash_value(varchar, varchar) TO GROUP :cm_ka_group;
GRANT EXECUTE ON FUNCTION hash_value(varchar, varchar) TO GROUP :admin_ka_group;

CREATE OR REPLACE FUNCTION get_age(myvalue date) 
RETURNS date 
LANGUAGE 'plpgsql' STABLE AS 
$$
BEGIN
    return date_trunc('year', myvalue::timestamp);
END;
$$;

REVOKE ALL ON FUNCTION get_age(date) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION get_age(date) TO GROUP :cm_ka_group;
GRANT EXECUTE ON FUNCTION get_age(date) TO GROUP :admin_ka_group;

CREATE OR REPLACE FUNCTION check_bool(myvalue integer, mybool integer) 
RETURNS integer 
LANGUAGE 'plpgsql' STABLE AS 
$$
BEGIN
    IF mybool = 1 THEN
        return myvalue;
    ELSE
        return NULL::integer;
    END IF;
END;
$$;

REVOKE ALL ON FUNCTION check_bool(integer, integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION check_bool(integer, integer) TO GROUP :cm_ka_group;
GRANT EXECUTE ON FUNCTION check_bool(integer, integer) TO GROUP :admin_ka_group;

CREATE OR REPLACE FUNCTION check_bool(myvalue varchar, mybool integer) 
RETURNS varchar 
LANGUAGE 'plpgsql' STABLE AS 
$$
BEGIN
    IF mybool = 1 THEN
        return myvalue;
    ELSE
        return NULL::varchar;
    END IF;
END;
$$;

REVOKE ALL ON FUNCTION check_bool(varchar, integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION check_bool(varchar, integer) TO GROUP :cm_ka_group;
GRANT EXECUTE ON FUNCTION check_bool(varchar, integer) TO GROUP :admin_ka_group;
