
BEGIN;
SET ROLE :adm_ka_owner;

-- Log the update
INSERT INTO db_history (beschreibung, zeitpunkt) 
	VALUES ('update_1_0_2/logbuch/01-tabellen.sql', current_timestamp);

-- Tabellen
CREATE TABLE ka_logbuch_art_tbl (
    id           SERIAL NOT NULL,
    beschreibung VARCHAR(80),
    CONSTRAINT ka_logbuch_art_tbl_pkey PRIMARY KEY (id)
);

INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES (-2, 'Sonstiges');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES (-1, 'Keine Angabe');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES ( 0, 'Automatischer Eintrag');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES ( 1, 'Gespräch mit dem Jugendlichen');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES ( 2, 'Telefonat mit dem Jugendlichen');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES ( 3, 'Begleitung des Jugendlichen zu externen Stellen');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES ( 4, 'Betreuung des Jugendlichen');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES ( 5, 'Schriftverkehr');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES ( 6, 'Gespräch/Telefonat mit Kooperationspartnern');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES ( 7, 'Gespräch/Telefonat mit Dritten');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES ( 8, 'Fallkonferenz (Fallbesprechung, Supervision)');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES ( 9, 'Erstgespräch');
INSERT INTO ka_logbuch_art_tbl (id, beschreibung) VALUES (10, 'Kompetenzfeststellung');

CREATE TABLE ka_logbuch_tbl (
    id         SERIAL NOT NULL,
    master_id  int4 REFERENCES master_tbl(id)         ON DELETE CASCADE,
    uuid_id    CHAR(36),
    datum      TIMESTAMP,
    bearbeiter int4 REFERENCES ka_benutzer_tbl(id)    ON DELETE SET DEFAULT,
    art        int4 REFERENCES ka_logbuch_art_tbl(id) ON DELETE SET DEFAULT,
    kurz_notiz VARCHAR(80),
    notiz      text,
    dauer      interval,
    UNIQUE (uuid_id),
    CONSTRAINT ka_logbuch_tbl_pkey PRIMARY KEY (id)
);

-- Views

CREATE OR REPLACE VIEW ka_logbuch_tbl_view 
    AS SELECT
        ka_logbuch_tbl.id         AS id,
        ka_logbuch_tbl.uuid_id    AS uuid_id,
        ka_logbuch_tbl.master_id  AS master_id,
        ka_logbuch_tbl.datum      AS datum,
        ka_logbuch_tbl.bearbeiter AS bearbeiter,
        ka_logbuch_tbl.art        AS art,
        ka_logbuch_tbl.kurz_notiz AS kurz_notiz,
        ka_logbuch_tbl.notiz      AS notiz,
        ka_logbuch_tbl.dauer      AS dauer
    FROM ka_logbuch_tbl
    INNER JOIN master_tbl ON
        ka_logbuch_tbl.master_id = master_tbl.id
    WHERE
        master_tbl.bearbeiter_id = getuserid()
        OR master_tbl.id IN (
            SELECT nm.master_id FROM nm_benutzer_master_tbl nm
            WHERE nm.benutzer_id IN (
                SELECT nm2.gruppe_id FROM nm_benutzer_gruppe_tbl nm2 
                WHERE nm2.benutzer_id = getuserid())
        )
        OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE ka_logbuch_update_rule
    AS ON UPDATE TO ka_logbuch_tbl_view 
	DO INSTEAD
        UPDATE ka_logbuch_tbl SET
            datum      = new.datum,
            bearbeiter = new.bearbeiter,
            art        = new.art,
            kurz_notiz = new.kurz_notiz,
            notiz      = new.notiz,
            dauer      = new.dauer
	    WHERE ka_logbuch_tbl.id = new.id;

CREATE OR REPLACE RULE ka_logbuch_delete_rule 
    AS ON DELETE TO  ka_logbuch_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_logbuch_insert_rule
    AS ON INSERT TO ka_logbuch_tbl_view DO INSTEAD NOTHING;

GRANT SELECT ON ka_logbuch_tbl_view TO GROUP :cm_ka_group;
GRANT UPDATE ON ka_logbuch_tbl_view TO GROUP :cm_ka_group;
GRANT SELECT ON ka_logbuch_tbl_view TO GROUP :admin_ka_group;

-- Funktionen

CREATE OR REPLACE FUNCTION create_logbuch_ds(mid integer, nuuid CHAR(36)) RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
AS $$
DECLARE
    lid integer;
BEGIN
    IF NOT (isEditorOrStandin(mid) OR pg_has_role('admin_ka','MEMBER')) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    INSERT INTO ka_logbuch_tbl(master_id, datum, bearbeiter) VALUES (mid, now(), getuserid());

    SELECT INTO lid currval('ka_logbuch_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE ka_logbuch_tbl SET uuid_id = nuuid WHERE id = lid;
    ELSE
        PERFORM set_uuid('ka_logbuch_tbl', lid);
    END IF;

    RETURN lid;
END;
$$;

REVOKE ALL    ON FUNCTION create_logbuch_ds(integer, CHAR(36)) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_logbuch_ds(integer, CHAR(36)) TO GROUP :cm_ka_group;
GRANT EXECUTE ON FUNCTION create_logbuch_ds(integer, CHAR(36)) TO GROUP :admin_ka_group;

CREATE OR REPLACE FUNCTION delete_logbuch_ds(tid integer) RETURNS void 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid master_id FROM ka_logbuch_tbl WHERE id = tid;
    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;
    DELETE FROM ka_logbuch_tbl WHERE id = tid;
END;
$$ ;

REVOKE ALL    ON FUNCTION delete_logbuch_ds(integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION delete_logbuch_ds(integer) TO GROUP :cm_ka_group;


CREATE OR REPLACE FUNCTION delete_all_logbuch(mid integer) 
RETURNS void 
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
AS $$
BEGIN
    DELETE FROM ka_logbuch_tbl WHERE master_id = mid;
END;
$$;

REVOKE ALL     ON FUNCTION delete_all_logbuch(integer) FROM PUBLIC;
GRANT  EXECUTE ON FUNCTION delete_all_logbuch(integer) TO GROUP :admin_ka_group;

-- Trigger fuer die letzte Veraenderung
CREATE TRIGGER t1_ka_logbuch_tbl_insert AFTER
    INSERT ON ka_logbuch_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_lastaccess_log();

CREATE TRIGGER t1_ka_logbuch_tbl_update AFTER
    UPDATE ON ka_logbuch_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_lastaccess_log();

CREATE TRIGGER t1_ka_logbuch_tbl_delete AFTER
    DELETE ON ka_logbuch_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_lastaccess_log();

-- Trigger fuer das automatische Anlegen eines Logbucheintrages
-- beim Anlegen eines Masterdatensatzes.

CREATE OR REPLACE FUNCTION master_ds_created_logbook() RETURNS trigger AS $master_ds_created_logbook$
DECLARE
    lid integer;
BEGIN
    SELECT INTO lid create_logbuch_ds(NEW.id, NULL);
    UPDATE ka_logbuch_tbl_view
        SET art = 0, notiz = 'Fallakte angelegt.'
    WHERE
        id = lid;
    RETURN NEW;
END;
$master_ds_created_logbook$ LANGUAGE plpgsql;

CREATE TRIGGER master_ds_created_logbook_trigger AFTER
    INSERT ON master_tbl
    FOR EACH ROW EXECUTE PROCEDURE master_ds_created_logbook();

--
-- Trigger, der nach Uebertragung der Fallakte an einen anderen
-- Bearbeiter den alten und neuen Bearbeiter eintraegt in das Logbuch eintraegt.
--
CREATE OR REPLACE FUNCTION master_ds_editor_changed() RETURNS trigger AS $master_ds_editor_changed$
DECLARE
    lid      integer;
    old_user RECORD;
    new_user RECORD;
BEGIN
    IF OLD.bearbeiter_id <> NEW.bearbeiter_id THEN

        SELECT INTO old_user * FROM ka_benutzer_tbl WHERE id = OLD.bearbeiter_id;
        SELECT INTO new_user * FROM ka_benutzer_tbl WHERE id = NEW.bearbeiter_id;

        SELECT INTO lid create_logbuch_ds(NEW.id, NULL);

        UPDATE ka_logbuch_tbl_view
            SET art = 0, 
            notiz = 'Fallakte übertragen von ' 
                    || coalesce(old_user.nachname, '-/-') || ', '
                    || coalesce(old_user.vorname,  '-/-') || ' an '
                    || coalesce(new_user.nachname, '-/-') || ', '
                    || coalesce(new_user.vorname,  '-/-')
        WHERE
            id = lid;
    END IF;
    RETURN NEW;
END;
$master_ds_editor_changed$ LANGUAGE plpgsql;

CREATE TRIGGER master_ds_editor_changed_trigger AFTER
    UPDATE ON master_tbl
    FOR EACH ROW EXECUTE PROCEDURE master_ds_editor_changed();


COMMIT;
