# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
# 
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS - 
# Web-Anwendungs-Server fuer Kompetenzagenturen).
# 
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
# 
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
# 
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
# 
# mpuls WASKA has been developed on behalf of the 
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and 
# European Social Fund resources.
#
# Authors:
# Torsten Irländer <torsten.irlaender@intevation.de>
#

import paste
import logging

from paste.httpexceptions           import HTTPNotFound
from waskaweb.lib.base              import *
from waskaweb.lib.renderer          import FormDataImpl, ViewRenderer, AidListRenderer
from waskaweb.model.case            import CaseFactory, LoadCaseNotExistsError 
from waskaweb.model.repeatgroup     import AidList 

log = logging.getLogger(__name__)

class CasebaseController(BaseController):

    def __before__(self):
        BaseController.__before__(self)
        self.navigation = session.get('navigation.tree')

    def index(self):
        raise HTTPNotFound()

    def _loadCase(self, id):
        factory   = CaseFactory()
        try:
            case = factory.loadById(id)
            return case
        except LoadCaseNotExistsError, err:
            self.showError()

    def getNavigation(self):
        if self.navigation: return self.navigation
        redirect_to(h.url_for(controller='/case_overview'))

    def _getRenderdFormElements(self, ti, ds_id, page_id=None, ro_mode=False, no_data=False):
        # Determine if we have an repeat group or a formular page
        what = int(ti.key) == 14 and "rg_aid_overview" or "formular"
        if what == "rg_aid_overview":
            navigation = self._getFormNavigation(selected_key="14")
            renderer = AidListRenderer(AidList(ds_id), ro_mode, no_data)
            content = renderer.render(ds_id, page_id) 
        else:
            navigation        = self._getFormNavigation(selected_key=ti.key)
            real_id           = ti.realId(ds_id)
            formdata          = self._getFormdata(ti, real_id, no_data)
            vr                = ViewRenderer(formdata, ro_mode)
            content           = vr.renderView(
                ti.key, 
                ds_id,  
                "/case/save", 
                showNext = not ti.nextKey() is None)
        return (content, navigation)

    def _getFormNavigation(self,selected_key=None):
        case = session.get('case')
        # TODO: In the following situation: Fresh login, no case selected.
        # select an appointment from blackboard. Find out why we need to get
        # the navigation tree from the session again. I thought that this is
        # alreday done in the _before_ method of this controller. 
        self.navigation = session.get('navigation.tree')
        if not case or not self.navigation:
            redirect_to(h.url_for(controller='/case_overview'))
        errors = case.getFormErrors()
        id     = case.id
        mode   = case.getMode()
        self.navigation.setErrors([err.page for err in errors.itervalues()])
        select_url = lambda t: '/case/%s/%s/%s' % (mode, id, t.key)
        toggle_url = lambda r: '/navigation/toggle/%s/%s' % (id, r.key)
        return self.navigation.render(select_url, toggle_url, selected_key) 

    def _getFormdata(self, treeItem, ds_id, no_data=False):
        case = session.get('case')
        if not case:
            redirect_to(controller="/case_overview")
        doc  = g.formedTree
        help = g.helpData
        page = doc.getPageStore(treeItem.page, ds_id, no_data=no_data)

        form_errors = case.getFormErrors()
        formdata = FormDataImpl(
            help=help, 
            page=page, 
            errors=form_errors, 
            nc=[page.getWidget()])
        return formdata

    def _setState(self, id, state):
        id      = self._checkInt(id)
        state   = self._checkInt(state)
        try:
            case  = self._loadCase(id)
            case_state = case.getState()
            case_state.setState(state)
            session_case = session.get('case')
            session_state = session_case.getState()
            session_state.setState(state)
            session.save()
            return True 
        except:
            return False

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
