# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
# 
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS - 
# Web-Anwendungs-Server fuer Kompetenzagenturen).
# 
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
# 
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
# 
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
# 
# mpuls WASKA has been developed on behalf of the 
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and 
# European Social Fund resources.
#
# Authors:
# Torsten Irländer <torsten.irlaender@intevation.de>
# Sascha L. Teichmann <teichmann@intevation.de>
#

import paste
import logging

from paste.httpexceptions          import HTTPNotFound
from waskaweb.lib.base             import *
from waskaweb.controllers.CaseBase import CasebaseController
from waskaweb.lib.security         import checkRole
from waskaweb.model.repeatgroup    import AidObject, AidObjectFactory, AidList 

log = logging.getLogger(__name__)

DELETE_CONFIRM_HEADER = u'''Unterstützungsangebot löschen?'''
DELETE_CONFIRM_EXPLAINATION = u'''Wollen Sie wirklich das Angebot löschen und die Daten unwiederbringlich verlieren?'''

class RgAidController(CasebaseController):

    def __before__(self):
        CasebaseController.__before__(self)

    def index(self):
        raise HTTPNotFound()

    @checkRole(('admin_ka', 'cm_ka', 'pt_dlr'))
    def show(self, rg_id, page_id):
        '''Returns the formular of an aid repeat group in show mode. The dataset is
        identified by it id (rg_id) and its "type" (page_id)'''
        rg_id     = self._checkInt(rg_id)
        page_id   = self._checkInt(page_id)
        ti        = self.getNavigation().getTreeItem(page_id)
        c.ds_id   = ti.realId(rg_id)
        c.page_id = page_id

        case_session = session.get('case')
        case_session.setMode("show")
        session.save()

        c.print_form = "show" # Which page to open in popup on printing?           

        content, navigation = self._getRenderdFormElements(ti, rg_id, page_id, ro_mode=True)
        c.formular = content
        c.form_navigation = navigation 
        return render('/casemanagement/formular.mako')

    @checkRole('cm_ka')
    def edit(self, rg_id, page_id):
        '''Returns the formular of an aid repeat group in edit mode. The dataset is
        identified by it id (rg_id) and its "type" (page_id)'''
        rg_id     = self._checkInt(rg_id)
        page_id   = self._checkInt(page_id)
        ti        = self.getNavigation().getTreeItem(page_id)
        c.ds_id   = ti.realId(rg_id)
        c.page_id = page_id

        case_session = session.get('case')
        case_session.setMode("edit")
        session.save()

        c.print_form = "show" # Which page to open in popup on printing?           

        content, navigation = self._getRenderdFormElements(ti, rg_id, page_id, ro_mode=False)
        c.formular = content
        c.form_navigation = navigation 
        return render('/casemanagement/formular.mako')

    @checkRole('cm_ka')
    def newAction(self):
        '''Creates a new aid repeat group. Returns the formular page of the new
        RG in edit-mode'''
        try:
            case_id  = request.params.getone('case_id')
            page_id  = request.params.getone('page_id')
            aid_type = request.params.getone('aid_type')
            factory  = AidObjectFactory()
            aid      = factory.create(int(aid_type))
            rg_id    = aid.create(int(case_id))
            if rg_id and aid.page_id:
                return self.edit(str(rg_id), aid.page_id)
            else:
                return "Error. Could not create new Aid"
        except KeyError:
            pass

    @checkRole('cm_ka')
    def delete(self, rg_id, ds_type, confirmed=False):
        '''Deletes new aid repeat group. If confirmed is false the user will
        get a confirmation dialog. This function return the overviewpage of the
        repeatgroups''' 
        rg_id     = self._checkInt(rg_id)
        ds_type   = self._checkInt(ds_type)
        confirmed = self._checkBool(confirmed)
        if confirmed == 1:
            factory  = AidObjectFactory()
            aid      = factory.load(int(ds_type), int(rg_id))
            if aid is None or aid.delete():
                # Redirect to the overview page: 14 should be the correct page key 
                c.success_for   = u'Angebot gelöscht!'
                c.success_text  = u'Bitte klicken Sie auf "OK", um fortzufahren.'
                c.url_ok        = "/case/edit/%s/14" % session.get('case').id
                return render('/casemanagement/dialogs/success_delete_aid.mako')
                #redirect_to('/case/edit/%s/14' % session.get('case').id) 
            else:
                return "Error! Could not delete ds."
        else:
            session_case  = session.get('case')
            c.confirm_for = DELETE_CONFIRM_HEADER 
            c.question    = DELETE_CONFIRM_EXPLAINATION 
            c.url_yes     = "/rg_aid/delete/%s/%s/1" % (rg_id, ds_type) 
            c.url_no      = "/case/edit/%s/14" % (session_case.id)
            return render('/casemanagement/dialogs/confirm_delete_aid.mako')

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
