# -*- coding: latin1 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
# 
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS - 
# Web-Anwendungs-Server fuer Kompetenzagenturen).
# 
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
# 
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
# 
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
# 
# mpuls WASKA has been developed on behalf of the 
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and 
# European Social Fund resources.
#
# Authors:
# Sascha L. Teichmann <teichmann@intevation.de>
#

from xml.dom.minidom import parseString
from xml.dom.ext     import Print
from cStringIO       import StringIO

import re

ANNOTATION_DIV_ID = re.compile("([a-zA-Z0-9_]+):([a-zA-Z]+):([a-zA-Z0-9_,]+)")

class Annotation:

    def __init__(self, kind, msg):
        self.kind = kind
        self.msg  = msg

    def __str__(self):
        return repr(self.kind) + ": " + repr(self.msg)

    def __repr__(self):
        return self.__str__()

def buildHelps(help_xhtml):
    helps = {}
    dom = None
    try:
        dom = parseString(help_xhtml)
        for div in dom.getElementsByTagName("div"):
            id = div.getAttribute("id")
            if not id: continue
            out = StringIO()
            Print(div, out, 'UTF-8')
            helps[id] = out.getvalue()
            out.close()
    finally:
        if dom: dom.unlink()

    return helps

def buildAnnotations(annotation_xhtml):
    annotations = {}
    if not annotation_xhtml:
        return annotations

    dom = None
    try:
        dom = parseString(annotation_xhtml)
        for div in dom.getElementsByTagName("div"):
            id = div.getAttribute("id")
            if not id: continue
            m = ANNOTATION_DIV_ID.match(id)
            if not m: continue
            id    = m.group(1)
            kind  = m.group(2)
            items = [item for item in m.group(3).split(",") if item]
            if not items: continue
            div.setAttribute("id", id)

            out = StringIO()
            Print(div, out, 'UTF-8')
            msg = out.getvalue()
            out.close()

            for item in items:
                annons = annotations.setdefault(item, [])
                annons.append(Annotation(kind, msg))
    finally:
        if dom: dom.unlink()

    return annotations

class HelpProvider:

    def __init__(self, help_xhtml, annotation_xhtml = None):

        self.helps       = buildHelps(help_xhtml)
        self.annotations = buildAnnotations(annotation_xhtml)

    def hasHelp(self, id):
        return self.helps.has_key(id)

    def getHelp(self, id, default=None):
        return self.helps.get(id, default)

    def hasAnnotations(self, id, kinds = ("required", )):
        annotations = self.annotations.get(id)
        if not annotations: return False
        for annotation in annotations:
            if annotation.kind in kinds:
                return True
        return False

    def getAnnotations(self, id, kinds = ("required", )):
        try:
            x = [a.msg for a in self.annotations[id] if a.kind in kinds]
        except KeyError:
            x = []
        return ''.join(x)

# vim:set ts=4 sw=4 si et sta sts=4:
