# -*- coding: latin1 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
# 
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS - 
# Web-Anwendungs-Server fuer Kompetenzagenturen).
# 
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
# 
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
# 
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
# 
# mpuls WASKA has been developed on behalf of the 
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and 
# European Social Fund resources.
#
# Authors:
# Sascha L. Teichmann <teichmann@intevation.de>
#

from nodecomponents import \
    NodeComponent, \
    Node, \
    Leaf, \
    VISIT_IGNORE_CHILDREN, \
    VISIT_CONTINUE

from expr           import Expr

import waskaweb.model.io.document

class FilteredNodeComponentProxy:
    def __init__(self, params):
        self.params = params

    def set(self, nc):
        self.nc = nc

    def getName(self):
        return self.nc.getName()

    def allWidgets(self):
        return [w for w in self.nc.allWidgets() if w.getName() in self.params]

class PageNode(Node):
    def __init__(self):
        Node.__init__(self)

    def allWidgets(self):

        widgets = []

        def visitor(nc):
            if isinstance(nc, Node):
                if isinstance(nc, (RadioNode, ChoiceNode)):
                    widgets.append(nc)
                    return VISIT_IGNORE_CHILDREN
                if isinstance(nc, GroupNode) and nc.isRepeat():
                    return VISIT_IGNORE_CHILDREN
            elif isinstance(nc, Leaf) and not isinstance(nc, 
                (RuleLeaf, ExternalChoiceListLeaf, InfoLeaf)):
                widgets.append(nc)

            return VISIT_CONTINUE

        self.visit(visitor)

        return widgets

class GroupNode(Node):
    def __init__(self):
        Node.__init__(self)
        self.attributes["repeat"]  = ""
        self.attributes["invisible"] = ""

    def isRepeat(self):
        return self.getAttribute("repeat") and True or False

    def setRepeat(self, repeat):
        repeat = repeat and "true" or ""
        self.setAttribute("repeat", repeat)

    def isInvisible(self):
        return self.getAttribute("invisible") and True or False

    def setInvisible(self, invisible):
        invisible = invisible and "true" or ""
        self.setAttribute("invisible", invisible)

    def allWidgets(self):

        widgets = []

        def visitor(nc):
            if isinstance(nc, Node):
                if isinstance(nc, (RadioNode, ChoiceNode)):
                    widgets.append(nc)
                    return VISIT_IGNORE_CHILDREN
                if isinstance(nc, GroupNode) and nc.isRepeat() and nc != self:
                    return VISIT_IGNORE_CHILDREN
            elif isinstance(nc, Leaf) and not isinstance(nc, 
                (RuleLeaf, ExternalChoiceListLeaf, InfoLeaf)):
                widgets.append(nc)

            return VISIT_CONTINUE

        self.visit(visitor)

        return widgets

class RadioNode(Node):
    def __init__(self):
        Node.__init__(self)
        self.attributes["selected"] = ""

    def getSelected(self):
        return self.getAttribute("selected")

    def setSelected(self, selected):
        self.setSelected("selected", selected)

class BoolLeaf(Leaf):
    def __init__(self):
        Leaf.__init__(self)
        self.attributes["checked"] = ""

    def getChecked(self):
        return self.getAttribute("checked")

    def setChecked(self, checked):
        self.setSelected("checked", checked)
    
class ExternalChoiceListLeaf(Leaf):
    def __init__(self):
        Leaf.__init__(self)
        self.choice = None

    def getChildren(self):
        if not self.choice:
            fname = self.getValue()
            if fname:
                try: # TODO: This loading code should be else where!
                    import waskaweb.model.io.document
                    document = waskaweb.model.io.document.openDocument(fname)
                    self.choice = document.findByClassAndName(ChoiceNode)
                except Exception, inst:
                    print str(inst)
                    pass
        if self.choice: 
            return self.choice.children
        return None

class InfoLeaf(Leaf):
    def __init__(self):
        Leaf.__init__(self)
        self.attributes["invisible"] = ""

    def isInvisible(self):
        return self.getAttribute("invisible") and True or False

    def setInvisible(self, invisible):
        invisible = invisible and "true" or ""
        self.setAttribute("invisible", invisible)

class TextAreaLeaf(Leaf):
    def __init__(self):
        Leaf.__init__(self)
        self.attributes["cols"] = ""
        self.attributes["rows"] = ""

    def getCols(self):
        return self.getAttribute("cols")

    def setCols(self, cols):
        return self.setAttribute("cols", cols)

    def getRows(self):
        return self.getAttribute("rows")

    def setRows(self, rows):
        return self.setAttribute("rows", rows)

class TextLeaf(Leaf):
    def __init__(self):
        Leaf.__init__(self)
        self.attributes["size"] = "40"
        self.attributes["maxlength"] = "60"

    def getSize(self):
        return self.getAttribute("size")

    def setSize(self, size):
        self.setAttribute("size", size)

    def getMaxLength(self):
        return self.getAttribute("maxlength")

    def setMaxLength(self, length):
        self.setAttribute("maxlength", length)

class ChoiceNode(Node):
    def __init__(self):
        Node.__init__(self)
        self.attributes["size"]     = "1"
        self.attributes["multiple"] = ""

    def getSize(self):
        return self.getAttribute("size")

    def setSize(self, size):
        self.setAttribute("size", size)

    def getMultiple(self):
        return self.getAttribute("multiple")

    def setMultiple(self, multiple):
        self.setAttribute("multiple", multiple)

    def getSelected(self):
        return self.getAttribute("selected")

    def setSelected(self, selected):
        self.setAttribute("selected", selected)

class IntLeaf(Leaf):
    def __init__(self):
        Leaf.__init__(self)
        self.attributes["minvalue"] = ""
        self.attributes["maxvalue"] = ""

    def getMinValue(self):
        return self.getAttribute("minvalue")

    def setMinValue(self, minValue):
        self.setAttribute("minvalue", minValue)

    def getMaxValue(self):
        return self.getAttribute("maxvalue")

    def setMaxValue(self, maxValue):
        self.setAttribute("maxvalue", maxValue)

class DateLeaf(Leaf):
    def __init__(self):
        Leaf.__init__(self)

class RuleLeaf(Leaf):
    def __init__(self):
        Leaf.__init__(self)
        self.attributes["expr"] = ""
        self.expr = None

    def shallowCopy(self):
        r = Leaf.shallowCopy(self)
        if self.expr: r.expr = self.expr.clone()
        else:         r.expr = None
        return r

    def setAttribute(self, attrib, value, broadcast=True):
        if attrib == "expr":
            newExpr = Expr(value)
            newExpr.compile()
            self.expr = newExpr
        Leaf.setAttribute(self, attrib, value, broadcast)

    def getExpr(self):
        return self.expr

# vim:set ts=4 sw=4 si et sta sts=4:
