# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
# 
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS - 
# Web-Anwendungs-Server fuer Kompetenzagenturen).
# 
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
# 
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
# 
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
# 
# mpuls WASKA has been developed on behalf of the 
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and 
# European Social Fund resources.
#
# Authors:
# Sascha L. Teichmann <sascha.teichmann@intevation.de>
#

import sys

UNKNOWN        = -1
CLEAR_START    =  0
CLEAR_ENDE     =  1
CM_START       =  2
CM_ENDE        =  3
NB_START       =  4
NB_ENDE        =  5
BERATUNG_START =  6
BERATUNG_ENDE  =  7

# dictionary for GUI
PHASE_DESCRIPTIONS = {
    UNKNOWN        : u"Unbekannt",
    CLEAR_START    : u"Eingangsanamnese läuft",
    CLEAR_ENDE     : u"Eingangsanamnese beendet",
    CM_START       : u"Case-Management läuft",
    CM_ENDE        : u"Case-Management beendet",
    NB_START       : u"Nachbetreuung läuft",
    NB_ENDE        : u"Nachbetreuung beendet",
    BERATUNG_START : u"Beratung läuft",
    BERATUNG_ENDE  : u"Beratung beendet"
}

# dictionary for symbolic handling like usage in FormEd trees.
PHASE_SYMBOLS = {
    UNKNOWN        : u"unbekannt",
    CLEAR_START    : u"clearing_start",
    CLEAR_ENDE     : u"clearing_ende",
    CM_START       : u"cm_start",
    CM_ENDE        : u"cm_ende",
    NB_START       : u"nachbetreuung_start",
    NB_ENDE        : u"nachbetreuung_ende",
    BERATUNG_START : u"beratung_start",
    BERATUNG_ENDE  : u"beratung_ende"
}

SYMBOLS_PHASE = dict([(i[1], i[0]) for i in PHASE_SYMBOLS.iteritems()])


PHASE_SUCCESSORS = {
    UNKNOWN       : [CLEAR_START],
    CLEAR_START   : [CLEAR_ENDE],
    CLEAR_ENDE    : [CM_START, BERATUNG_START],
    CM_START      : [CM_ENDE],
    BERATUNG_START: [BERATUNG_ENDE],
    BERATUNG_ENDE : [],
    CM_ENDE       : [NB_START],
    NB_START      : [NB_ENDE],
    NB_ENDE       : []
}

def phase_description(phase):
    return PHASE_DESCRIPTIONS.get(phase, "-/-")

def phase_symbol(phase):
    return PHASE_SYMBOLS[phase]

def symbol_phase(symbol):
    return SYMBOLS_PHASE[symbol]

def phase_successors(phase):
    try:
        return PHASE_SUCCESSORS[phase]
    except KeyError:
        return []

def phase_predecessors(phase):
    preds = set()
    for k, v in PHASE_SUCCESSORS.iteritems():
        if phase in v:
            preds.add(k)
    return list(preds)

def phase_predecessors_path(phase, path=None, include_root=False):
    if not path:
        path = []
        if include_root:
            path.append(phase)
    for p in phase_predecessors(phase):
        path.append(p)
        phase_predecessors_path(p, path=path)
    return path

def phase_successors_path(phase, path=None, include_root=False):
    if not path:
        path = []
        if include_root:
            path.append(phase)
    for p in phase_successors(phase):
        path.append(p)
        phase_successors_path(p, path=path)
    return path

def phase_neighbors(phase1, phase2):
    try:
        return phase2 in PHASE_SUCCESSORS[phase1] or phase1 in PHASE_SUCCESSORS[phase2]
    except KeyError, ke:
        print >> sys.stderr, "missing key in phase_neighbors:", ke
        return False

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf-8:
