# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
# 
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS - 
# Web-Anwendungs-Server fuer Kompetenzagenturen).
# 
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
# 
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
# 
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
# 
# mpuls WASKA has been developed on behalf of the 
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and 
# European Social Fund resources.
# Authors:
#
# Sascha L. Teichmann <sascha.teichmann@intevation.de>
#

from waskaweb.model.phases           import *
from waskaweb.model.phase_transition import *

from waskaweb.model.data             import PageNode

IS_FILLED = \
"""(%(name)s IS NOT NULL
    AND (get_default_value('master_tbl', '%(name)s') IS NULL
        OR %(name)s != get_default_value('master_tbl', '%(name)s')))
"""

IS_NOT_FILLED = \
"""(NOT (%(name)s IS NOT NULL
    AND (get_default_value('master_tbl', '%(name)s') IS NULL
        OR %(name)s != get_default_value('master_tbl', '%(name)s'))))
"""

SELECT = \
''' SELECT %s FROM master_tbl_view WHERE id = %%(id)s
'''

def description(nc):
    p = nc
    while p:
        desc = p.getDescription()
        if desc:
            desc = desc.strip()
            if desc: return desc
        p = p.parent
    return nc.getName()

def page(nc):
    p = nc.parent
    while p:
        if isinstance(p, PageNode):
            return p
        p = p.parent
    return p

def lookup_item(item, navigation, link):
    if navigation and link:
        p = page(item)
        if p:
            ti = navigation.findTreeItemByPageName(p.getName())
            if ti: return link(ti)
    return '"/case/required/%s" target="_blank"' % item.getName()

class RequiredFields:

    def __init__(self):
        self.phases = {}

    def extractRequiredFields(self, document):

        phases, times = {}, {}

        all, all_times = [], []
        
        for nc in document.walk():
            flags = nc.getFlags()
            if flags is None: continue
            for pair in flags.split(';'):
                pair = pair.strip()
                if not pair: continue
                p = [p.strip() for p in pair.split(':')]
                if len(p) > 1:
                    kind, phase = p[0], p[1]
                    if kind == "required":
                        for p in [p.strip() for p in phase.split(',')]:
                            if p: phases.setdefault(p, []).append([nc, None])
                    elif kind == "time":
                        for p in [p.strip() for p in phase.split(',')]:
                            if p: times.setdefault(p, []).append([nc, None])
                else:
                    # TODO: Fix formed tree
                    if p[0] == "required":
                        all.append([nc, None])
                    elif p[0] == "time":
                        all_times.append([nc, None])

        for a in all:
            for v in phases.itervalues():
                v.append(a)

        for t in all_times:
            for v in times.itervalues():
                v.append(t)

        for v in phases.itervalues():
            v.sort(cmp=lambda a, b: cmp(description(a[0]), description(b[0])))

        self.phases = phases
        self.times  = times
        return phases

    def loadTimes(self, ds_id, cur):
        fields = {}
        idx = 0
        for f in self.times.itervalues():
            for l in f:
                name = l[0].getName()
                if not fields.has_key(name):
                    fields[name] = idx
                    idx += 1

        all = fields.items()

        if not all: return True

        all.sort(cmp = lambda a, b: cmp(a[1], b[1]))

        select = SELECT % ', '.join([a[0] for a in all])

        cur.execute(select, { 'id': ds_id })
        row = cur.fetchone()
        if not row: return False

        for a in all:
            fields[a[0]] = row[a[1]]

        for a in self.times.itervalues():
            for b in a:
                b[1] = fields.get(b[0].getName())

        return True

    def loadFromDatabase(self, ds_id, cur):
        return self.loadFilled(ds_id, cur) and self.loadTimes(ds_id, cur) and True or False


    def getSQLWhereClauses(self):
        """ returns list of tuples (<symbol of phase>, <sql where clause>). The
        where clause will return cases with inconsistencies for the current
        state."""
        clauses = []
        # Iterate over all phases
        for symbol in self.phases.keys():

            # for each phase build a dict of predecessor of the current phase
            predecessor_phases = {}
            phase_id = symbol_phase(symbol)
            for id in phase_predecessors_path(phase_id, include_root=True):
                if id == -1: continue #ignore "unknown" phase
                predecessor_phases[phase_symbol(id)] = self.phases.get(phase_symbol(id))
            # now build the where clause to check if the current phase is
            # consistent.
            where_parts = []
            for s, v in predecessor_phases.iteritems():
                try:
                    part = " OR ".join([IS_NOT_FILLED % { 'name': name} for name in [i[0].getName() for i in v]])
                except:
                    pass
                where_parts.append(part)
            where = "(phase = %s AND ((%s))) " % (phase_id, " OR ".join(where_parts))
            clauses.append((symbol, where))
        return clauses

    def loadFilled(self, ds_id, cur):
        fields = {}
        idx = 0
        for f in self.phases.itervalues():
            for l in f:
                name = l[0].getName()
                if not fields.has_key(name):
                    fields[name] = idx
                    idx += 1

        all = fields.items()

        if not all: return True

        all.sort(cmp = lambda a, b: cmp(a[1], b[1]))

        select_terms = []
        for a in all:
            select_terms.append(IS_FILLED % { 'name': a[0] })

        select = SELECT % ', '.join(select_terms)

        cur.execute(select, { 'id': ds_id })
        row = cur.fetchone()
        if not row: return False

        for a in all:
            fields[a[0]] = row[a[1]]

        for a in self.phases.itervalues():
            for b in a:
                b[1] = fields.get(b[0].getName())

        return True

    def getPhase(self, phase):
        return self.phases.get(phase)


    def getPhaseTime(self, phase):
        try:
            return self.times[phase][0][1]
        except KeyError:
            return None

    def isPhaseComplete(self, phase):
        try:
            for l in self.phases[phase]:
                if not l[1]:
                    return False
        except KeyError:
            return False

        return True

    def getLinkListForPhase(self, phase, navigation = None, link = None):
        try:
            phase = self.phases[phase]
        except KeyError:
            return u""

        out = []

        for a in phase:
            warn = (not a[1]) and ' class="required_missing"' or ""
            l = '<a href=%s %s>%s</a>' % (
                lookup_item(a[0], navigation, link),
                warn, 
                description(a[0]))
            out.append(l)

        return ',\n'.join(out)

PAIRS = (
    (CLEAR_START,    CLEAR_ENDE,    u"Eingangs-<wbr />Anamnese"),
    (CM_START,       CM_ENDE,       u"Case-<wbr />Management"),
    (BERATUNG_START, BERATUNG_ENDE, u"Beratung"),
    (NB_START,       NB_ENDE,       u"Nach&shy;betreuung"))

def phases_pairs(required_fields, current_phase, navigation = None, link = None):

    pairs = []

    all = {}

    symbol = phase_symbol(current_phase)
    cur_phase = Phase(
                current_phase,
                True,
                required_fields.isPhaseComplete(symbol),
                required_fields.getPhaseTime(symbol),
                required_fields.getLinkListForPhase(symbol, navigation, link))

    for p1, p2, description in PAIRS:

        symbol = phase_symbol(p1)

        start = Phase(
            p1,
            p1 == current_phase,
            required_fields.isPhaseComplete(symbol),
            required_fields.getPhaseTime(symbol),
            required_fields.getLinkListForPhase(symbol, navigation, link))

        all[p1] = start

        symbol = phase_symbol(p2)

        ende = Phase(
            p2,
            p2 == current_phase,
            required_fields.isPhaseComplete(symbol),
            required_fields.getPhaseTime(symbol),
            required_fields.getLinkListForPhase(symbol, navigation, link))

        all[p2] = ende


        # Only add phases which are on the path of the current phase
        #pairs.append(PhasePair(start, ende, description))
        if cur_phase.hasSuccessor(start) or cur_phase.hasPredecessor(start) or current_phase == p1: 
            pairs.append(PhasePair(start, ende, description))

    for p in all.itervalues():
        for pred in phase_predecessors(p.phase):
            try:
                p.addPredecessor(all[pred])
            except KeyError:
                pass

    return pairs

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
